// Copyright (c) 2023 ETH Zurich.
//                    All rights reserved.
//
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
//
// main author: Robert Gerstenberger

#ifndef __NOD_PROPERTY_H
#define __NOD_PROPERTY_H

/**
  a check for the maximum size of those meta data fields is included in the
  code of NOD_LinearScanningAddProperty,
  NOD_LinearScanningRemoveSpecificProperty,
  NOD_LinearScanningUpdateSingleEntityProperty,
  NOD_LinearScanningUpdateSpecificProperty and
  NOD_LinearScanningSetSingleEntityProperty, so any changes to
  NOD_PropertyHandle and NOD_PropertyRecordSize should update that code as well
 */
typedef uint8_t   NOD_PropertyHandle;
/**
  TODO: change this again to uint16_t
 */
typedef uint64_t  NOD_PropertyRecordSize;

#define NOD_PROPERTY_HANDLE_SIZE      sizeof(NOD_PropertyHandle)
#define NOD_PROPERTY_RECORD_SIZE      sizeof(NOD_PropertyRecordSize)
#define NOD_PROPERTY_METADATA_SIZE    (NOD_PROPERTY_HANDLE_SIZE + NOD_PROPERTY_RECORD_SIZE)
#define NOD_PROPERTY_OFFSET_PRIMARY   0
#define NOD_PROPERTY_OFFSET_BLOCK     0

#define NOD_PROPERTY_EMPTY    0
#define NOD_PROPERTY_LAST     1
#define NOD_PROPERTY_LABEL    2

/**
  TODO: Also use a return type, where appropriate, for the label functions?
 */
int NOD_LinearScanningAddProperty( GDI_PropertyType ptype, const void* value, size_t count, GDI_VertexHolder vertex );
int NOD_LinearScanningFindAllProperties( void* buf, size_t buf_count, size_t* buf_resultcount, size_t* array_of_offsets, size_t offset_count, size_t* offset_resultcount, GDI_PropertyType ptype, GDI_VertexHolder vertex );
int NOD_LinearScanningFindAllPropertyTypes( GDI_VertexHolder vertex, GDI_PropertyType* ptypes, size_t count, size_t* resultcount );
void NOD_LinearScanningNumProperties( GDI_VertexHolder vertex, GDI_PropertyType ptype, size_t* resultcount, size_t* element_resultcount );
void NOD_LinearScanningNumPropertyTypes( GDI_VertexHolder vertex, size_t* resultcount );
void NOD_LinearScanningRemoveProperties( GDI_PropertyType ptype, GDI_VertexHolder vertex, bool* found_flag );
void NOD_LinearScanningRemoveSpecificProperty( GDI_PropertyType ptype, const void* value, size_t count, GDI_VertexHolder vertex, bool* found_flag );
void NOD_LinearScanningSetSingleEntityProperty( GDI_PropertyType ptype, const void* value, size_t count, GDI_VertexHolder vertex, bool* found_flag );
int NOD_LinearScanningUpdateSingleEntityProperty( GDI_PropertyType ptype, const void* value, size_t count, GDI_VertexHolder vertex );
int NOD_LinearScanningUpdateSpecificProperty( GDI_PropertyType ptype, const void* old_value, size_t old_count, const void* new_value, size_t new_count, GDI_VertexHolder vertex );

int NOD_LinearScanningFindAllLabels( GDI_VertexHolder vertex, GDI_Label* labels, size_t count, size_t* resultcount );
void NOD_LinearScanningInitPropertyList( GDI_VertexHolder vertex );
void NOD_LinearScanningInsertLabel( GDI_Label label, GDI_VertexHolder vertex, bool* found_flag );
void NOD_LinearScanningNumLabels( GDI_VertexHolder vertex, size_t* resultcount );
void NOD_LinearScanningRemoveLabel( GDI_Label label, GDI_VertexHolder vertex, bool* found_flag );

#endif // #ifndef __NOD_PROPERTY_H
